/*
	    Universe Objects Management Math Utilities
 */

#ifndef UNVMATH_H
#define UNVMATH_H

#include <sys/types.h>
#include "objects.h"


extern float RADTODEG(const float radians);
extern float DEGTORAD(const float degrees);
extern float SANITIZERADIANS(const float radians);
extern float SANITIZEDEGREES(const float degrees);

/* Polar To Coordinate Conversion */
extern float MuPolarRotX(const float theta, const float r);
extern float MuPolarRotY(const float theta, const float r);

/* Vector Unitlization */
extern void MuSetUnitVector2D(
	sw_vector_compoent_struct *v,
	const float bearing
);
extern void MuSetUnitVector3D(
	sw_vector_compoent_struct *v,
	const float bearing, const float pitch
);

/* Angle & Vector Calculations */
extern float MuAngleDelta(const float theta_to, const float theta_cur);
extern void MuVectorAdd(
	const float v1_dir, const float v1_mag,
	const float v2_dir, const float v2_mag,
	float *v_dir_rtn, float *v_mag_rtn
);
extern float MuCoordinateDeltaVector(const float dx, const float dy);

/* 3D Distance */
extern float Mu3DDistance(
	const float dx, const float dy, const float dz
);

/* Sector Checking */
extern int Mu3DInSectorPtr(
	xsw_object_struct *obj_ptr,
	const long sect_x, const long sect_y, const long sect_z
);
extern int Mu3DInSameSector(
	xsw_object_struct **xsw_object, const int total_objects,
	const int obj_num1, const int obj_num2
);
extern int Mu3DInSameSectorPtr(
	xsw_object_struct *obj1, xsw_object_struct *obj2
);

/* Spherical Contact */
extern int Mu3DInContact(
	xsw_object_struct **xsw_object, const int total_objects,
	const int obj_num1, const int obj_num2
);
extern int Mu3DInContactPtr(
	xsw_object_struct *obj1, xsw_object_struct *obj2
);

/* Vector Contact */
extern int Mu3DInVectorContact(
	xsw_object_struct **xsw_object, const int total_objects,
	const int obj_num1, const int obj_num2,
	const float heading,		/* Object 1 to object 2 */
	const float heading_variance,
	const float range		/* In screen units */
);

/* Range Contact */
extern int Mu3DInRange(
	xsw_object_struct **xsw_object, const int total_objects,
	const int obj_num1, const int obj_num2,
	const float distance		/* In real units */
);
extern int Mu3DInRangePtr(
        xsw_object_struct *obj, xsw_object_struct *tar_obj,
	const float distance		/* In real units */
);

/* Maximum range by velocity and time */
extern float MuMaxRangeByVelocity(const float v_max, const time_t t_ms);


#endif	/* UNVMATH_H */
