#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "../include/reality.h"
#include "../include/unvmath.h"

#include "particle.h"
#include "net.h"
#include "swterm.h"


void BridgeDrawTitle(CDisplay *display, swt_bridge_struct *bw);
void BridgeDrawHelp(CDisplay *display, swt_bridge_struct *bw);

void BridgeDrawClearVS(CDisplay *display, swt_bridge_struct *bw);
void BridgeDrawPlayerStats(
	CDisplay *display, swt_bridge_struct *bw,
	const int obj_num
);
void BridgeDrawSubjectStats(
	CDisplay *display, swt_bridge_struct *bw,
	const int obj_num
);

static void BridgeDrawVSObject(
	CDisplay *display, swt_bridge_struct *bw, Window w,
	xsw_object_struct *camera_obj,
	xsw_object_struct *camera_locked_obj,
	xsw_object_struct *obj,
	const int x, const int y,
	const time_t cur_ms, const time_t cur_sec, const float tc
);
static void BridgeDrawVSParticle(
	CDisplay *display, swt_bridge_struct *bw, Window w,
	xsw_particle_struct *p,
	const int x, const int y,
	const time_t cur_ms, const time_t cur_sec, const float tc
);
void BridgeDrawVSObjects(
	CDisplay *display, swt_bridge_struct *bw,
	const time_t cur_ms, const time_t cur_sec, const float tc
);

void BridgeDrawMessages(CDisplay *display, swt_bridge_struct *bw);
void BridgeDrawStatusBar(CDisplay *display, swt_bridge_struct *bw);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


#define RADTODEG(r)	((r) * 180 / PI)


/* Bar color coefficient values */
#define BW_WARNING_COEFF                0.58
#define BW_DANGER_COEFF                 0.30


/*
 *	Redraws the bridge title.
 */
void BridgeDrawTitle(CDisplay *display, swt_bridge_struct *bw)
{
	int x, y;
	unsigned int width, height;
	char *s;
	Window w;
	xsw_net_parms_struct *net = &net_parms;

	if(display == NULL)
	    return;

	w = display->root_win;

	width = CWRootWidth(display);
	height = CWRootHeight(display);

	CWSetColor(display, w, swt_color.title, CColorFlagBold | CColorFlagReverse);
	CWClearArea(display, w, 0, 0, width, 1);

	switch(net->connection_state)
	{
	  case XSW_CON_STATE_CONNECTED:
	  case XSW_CON_STATE_CLOSE_WAIT:
	    if(!STRISEMPTY(net->title))
	    {
		if(!STRISEMPTY(net->login_name))
		{
		    s = (char *)malloc(
			STRLEN(PROG_NAME_FULL) +
			STRLEN(net->login_name) +
			STRLEN(net->title) +
			80
		    );
		    sprintf(
			s,
			"%s: %s - %s",
			PROG_NAME_FULL, net->login_name, net->title
		    );
		}
		else
		{
		    s = (char *)malloc(
			STRLEN(PROG_NAME_FULL) +
			STRLEN(net->title) + 
			80 
		    );
		    sprintf(
			s,
			"%s: %s",
			PROG_NAME_FULL, net->title
		    );
		}
	    }
	    else
	    {
		if(!STRISEMPTY(net->login_name))
		{
		    s = (char *)malloc(
			STRLEN(PROG_NAME_FULL) +
			STRLEN(net->login_name) +
			STRLEN(net->address) + 
			80 
		    );
		    sprintf(
			s,
			"%s: %s - %s",
			PROG_NAME_FULL, net->login_name, net->address
		    );
		}
		else
		{
		    s = (char *)malloc(
			STRLEN(PROG_NAME_FULL) +
			STRLEN(net->address) + 
			80
		    );
		    sprintf(
			s,
			"%s: %s",
			PROG_NAME_FULL, net->address
		    );
		}     
	    }
	    break;

	  default:
	    s = (char *)malloc(
		STRLEN(PROG_NAME_FULL) + 80
	    );
	    sprintf(
		s,
		"%s: Untitled",
		PROG_NAME_FULL
	    );
	    break;
	}

	x = ((int)width - STRLEN(s)) / 2;
	y = 0;
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    s
	);
	free(s);
}

/*
 *	Draws help keys.
 */
void BridgeDrawHelp(CDisplay *display, swt_bridge_struct *bw)
{
	int x, y;
	Window w;

	if(display == NULL)
	    return;

	w = display->root_win;

	x = 1;
	y = 2;

	CWSetColor(display, w, swt_color.standard, CColorFlagBold);

	CWDrawStringPosition(
	    display, w,
	    x, y,
"F9 = Connect.                       F10 = Disconnect."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"F11 = Refresh.                      F12 = Connect last."  
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"E = Server command.                 X = Exit this program."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"A = Auto network tunning on/off.    [ ] = Adjust network interval."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"UP DOWN = Throttle.                 LEFT RIGHT = Heading."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"B = Retro thrusters.                "
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"MINUS PLUS = Viewscreen zoom.       , . = Select weapon."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"W = Weapons offline/online.         SPACE = Fire weapon."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"TAB = Lock on the `next' object.    N = Lock on the `nearest' object."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"U = Unlock weapons."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"K = Set the com channel.            ' = Send message."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"H = Send hail.                      S = Raise/lower the shields."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"F = Set the shield frequency.       Y = Set the weapons frequency."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"C = Turn damage control on/off.     D = Initiate/abort auto-destruct."
	);

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"I = Set the object to intercept.    Z = Raise/lower the cloak."
	); 

	y += 1;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"L = Turn the lights on/off.         P = Turn the strobes on/off."
	);


	y += 2;
	CWDrawStringPosition(
	    display, w,
	    x, y,
"Press F1 to exit this screen."
	);
}


/*
 *	Clears the bridge window viewscreen area.
 */
void BridgeDrawClearVS(CDisplay *display, swt_bridge_struct *bw)
{
	unsigned int width, height, cw, ch;
	Window w;

	if((display == NULL) || (bw == NULL))
	    return;

	w = display->root_win;

	width = CWRootWidth(display);
	height = CWRootHeight(display);

	cw = width;
	ch = (unsigned int)MAX(
	    ((int)height - 1 - bw->msg_lines_visible - 1),
	    0
	);

	/* If the prompt is mapped then do not clear the last line
	 * which is reserved for the prompt
	 */
	if(bw->prompt_mode != SWT_PROMPT_MODE_NONE)
	    ch = (unsigned int)MAX(((int)ch - 1), 0);

	CWSetColor(display, w, swt_color.standard, 0);
	CWClearArea(
	    display,
	    w,
	    0, 1,
	    cw, ch
	);
}



/*
 *	Draws player stats.
 *
 *	Does not clear or refresh the bridge window, the bridge window
 *	is assumed to have been already cleared.
 */
void BridgeDrawPlayerStats(
	CDisplay *display, swt_bridge_struct *bw,
	const int obj_num
)
{
	int i, x, y, xc, wc, hc, len, std_color_num;
	unsigned int width, height;
	float throttle;
	const char *name;
	char ns[40];
	xsw_object_struct *obj;
	int sel_wep_num;
	xsw_weapon_struct *wep;
	Window w;
	sw_legend_struct *legend = &sw_legend;

	if((display == NULL) || (bw == NULL))
	    return;

	w = display->root_win;

	width = CWRootWidth(display);
	height = CWRootHeight(display);
	wc = 40;
	hc = (int)height - 5;

	if(DBIsObjectGarbage(obj_num))
	    return;

	obj = xsw_object[obj_num];


	/* Begin drawing */
	std_color_num = swt_color.standard;
	x = 1;
	y = 1;

#define SET_COLOR(_c_)		CWSetColor(display, w, (_c_), 0)
#define SET_COLOR_BOLD(_c_)	CWSetColor(display, w, (_c_), CColorFlagBold)
#define DRAW_STRING(_s_)	CWDrawString(display, w, (_s_))

	/* Player name */
	name = DBGetFormalNameStr(obj_num);
	SET_COLOR_BOLD(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    name
	);   

	/* Com channel */
	y += 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Com Channel: "
	);
	sprintf(ns, "%.2f", (float)obj->com_channel / 100.0f);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);

	SET_COLOR(std_color_num);
	DRAW_STRING("  Wep Freq: ");
	sprintf(ns, "%.2f", player_values.weapon_freq);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);

	/* Hit points */
	y += 1;
	i = (int)((obj->hp_max > 0.0f) ?
	    ((float)obj->hp / (float)obj->hp_max * 100.0f) :
	    0.0f
	);
	SET_COLOR(std_color_num);  
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "HP: "
	);
	sprintf(ns, "%.0f", obj->hp);
	if(i <= (int)(SWT_HP_DANGER_COEFF * 100.0f))
	    SET_COLOR_BOLD(swt_color.danger);
	else if(i <= (int)(SWT_HP_WARNING_COEFF * 100.0f)) 
	    SET_COLOR_BOLD(swt_color.warning);
	else
	    SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING("(");
	sprintf(ns, "%.0f", obj->hp_max);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(") ");
	sprintf(ns, "%i%%%%", i);
	if(i <= (int)(SWT_HP_DANGER_COEFF * 100.0f)) 
	    SET_COLOR_BOLD(swt_color.danger);         
	else if(i <= (int)(SWT_HP_WARNING_COEFF * 100.0f)) 
	    SET_COLOR_BOLD(swt_color.warning);       
	else
	    SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);

	if(obj->damage_control == SW_DMGCTL_STATE_ON)
	{
	    SET_COLOR(std_color_num);
	    DRAW_STRING(" [");
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING("DmgCtl");
	    SET_COLOR(std_color_num);
	    DRAW_STRING("]");
	}

	/* Core power */
	y += 1;
	i = (int)((obj->power_max > 0.0f) ?
	    ((float)obj->power / (float)obj->power_max * 100.0f) :
	    0.0f
	);
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "CP: "
	);
	sprintf(ns, "%.0f", obj->power);
	if(i <= (int)(SWT_POWER_DANGER_COEFF * 100.0f)) 
	    SET_COLOR_BOLD(swt_color.danger);         
	else if(i <= (int)(SWT_POWER_WARNING_COEFF * 100.0f)) 
	    SET_COLOR_BOLD(swt_color.warning);       
	else
	    SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING("(");
	sprintf(ns, "%.0f", obj->power_max);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(")");

	/* Shield state and freq */
	y += 1;
	if((obj->shield_state == SW_SHIELD_STATE_UP) &&
	   (obj->power_max > 0.0f)
	)
	{
	    i = (int)(obj->power / obj->power_max * 100.0f);
	    SET_COLOR(std_color_num);
	    CWDrawStringPosition(
		display, w,
		x, y,
		"SH: "
	    );
	    sprintf(ns, "%i%%%%", i);
	    if(i <= (int)(SWT_POWER_DANGER_COEFF * 100.0f))
		SET_COLOR_BOLD(swt_color.danger);
	    else if(i <= (int)(SWT_POWER_WARNING_COEFF * 100.0f))
		SET_COLOR_BOLD(swt_color.warning);
	    else
		SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(" @ ");
	    sprintf(ns, "%.2f", obj->shield_frequency);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	}

	/* Visibility and cloak state */
	y += 1;
	i = (int)(DBGetObjectVisibilityPtr(obj) * 100.0f);
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Vis: "
	);
	sprintf(ns, "%i%%%%", i);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	if(obj->cloak_state == SW_CLOAK_STATE_UP)
	{
	    SET_COLOR(std_color_num);
	    DRAW_STRING("  [");
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING("Cloak");
	    SET_COLOR(std_color_num);
	    DRAW_STRING("]");
	}

	/* Engine and antimatter */
	y += 1;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Eng: "
	);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(
	    (obj->engine_state == SW_ENGINE_STATE_ON) ? "On" : "Off"
	);
	SET_COLOR(std_color_num);
	DRAW_STRING("  AM: ");
	sprintf(ns, "%.2f", obj->antimatter);
	if((legend->antimatter_low_warning > 0.0f) &&
	   (obj->antimatter_max > legend->antimatter_low_warning)
	)
	{
	    const float	v_warning = legend->antimatter_low_warning,
			v_danger = v_warning / 2.0f;
	    if(obj->antimatter > v_warning)
		SET_COLOR_BOLD(std_color_num);
	    else if(obj->antimatter > v_danger)
		SET_COLOR_BOLD(swt_color.warning);
	    else
		SET_COLOR_BOLD(swt_color.danger);
	}
	else
	{
	    const float c = (obj->antimatter_max > 0.0f) ?
		(obj->antimatter / obj->antimatter_max) : 0.0f;
	    if(c <= SWT_ANTIMATTER_DANGER_COEFF)
		SET_COLOR_BOLD(swt_color.danger);
	    else if(c <= SWT_ANTIMATTER_WARNING_COEFF)
		SET_COLOR_BOLD(swt_color.warning);
	    else
		SET_COLOR_BOLD(std_color_num);
	}
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING("(");
	sprintf(ns, "%.2f", obj->antimatter_max);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(")");

	/* Intercept */
	y += 1;
	if(!DBIsObjectGarbage(obj->intercepting_object))
	{
	    SET_COLOR(std_color_num);
	    CWDrawStringPosition(
		display, w,
		x, y,
		"I: "
	    );
	    name = DBGetFormalNameStr(obj->intercepting_object);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(name);
	}

	/* Heading, velocity and position */
	y += 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "HD: "
	);
	sprintf(ns, "%.2f'", RADTODEG(obj->heading));
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING("  Vel: ");
	sprintf(
	    ns,
	    "%.2f au/s",
	    obj->velocity * legend->ru_to_au *
		1000.0f / (float)SW_CYCLE_LAPSE_MS
	);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);

	y += 1;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Sect: "
	);
	sprintf(ns, "%ld", obj->sect_x);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(ns, "%ld", obj->sect_y);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(ns, "%ld", obj->sect_z);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);

	y += 1;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Pos: "
	);
	sprintf(ns, "%.2f", obj->x);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(ns, "%.2f", obj->y);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(ns, "%.2f", obj->z);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(ns);

	y += 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "[----+----]"
	);

	SET_COLOR_BOLD(std_color_num);
	len = STRLEN("[----+----]") - 2;
	throttle = (xsw_gctl != NULL) ? xsw_gctl->throttle : 0.0f;
	xc = (int)(((throttle + 1.0f) / 2.0f) * len) + 1;
	if(xc < 1)
	    xc = 1;
	if(xc >= (len + 1))
	    xc = (len + 1) - 1;
	CWDrawCharacter(
	    display, w,
	    x + xc, y,
	    '|'
	);

	sprintf(ns, "%.0f%%", throttle * 100.0f);
	CWDrawStringPosition(
	    display, w,
	    x + 14, y,
	    ns
	);


	/* Selected weapon */
	y += 2;
	while(player_values.weapons_online)
	{
	    sel_wep_num = obj->selected_weapon;
	    if((sel_wep_num < 0) ||
	       (sel_wep_num >= obj->total_weapons)
	    )
		break;

	    wep = obj->weapons[sel_wep_num];
	    if(wep == NULL)
		break;

	    SET_COLOR(std_color_num);
	    CWDrawStringPosition(
		display, w,
		x, y,
		"Wep "
	    );
	    sprintf(ns, "%i", sel_wep_num + 1);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(": ");
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(wep->name);

	    switch(wep->emission_type)
	    {
	      case SW_WEP_EMISSION_PROJECTILE:
		SET_COLOR(std_color_num);
		DRAW_STRING(": ");
		sprintf(ns, "%i", wep->amount);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(ns);
		SET_COLOR(std_color_num);
		DRAW_STRING("(");
		sprintf(ns, "%i", wep->max);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(ns);
		SET_COLOR(std_color_num);
		DRAW_STRING(")");
		break;

	      default:
		i = (int)((obj->power_max > 0.0f) ?
		    (obj->power / obj->power_max * 100.0f) :
		    0.0f
		);
		SET_COLOR(std_color_num);
		DRAW_STRING(": ");
		sprintf(ns, "%i%%%%", i);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(ns);             
		break;
	    }

	    break;
	}


	/* Viewscreen zoom (lower left) */
	x = 1;
	y = (int)height - 1 - bw->msg_lines_visible - 1 - 1;

	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Scanner Range: "
	);
	if(obj->loc_type == SW_LOC_TYPE_NEBULA)
	{
	    sprintf(
		ns,
		"%.2f au",
		obj->scanner_range * legend->ru_to_au *
		    bw->vs_zoom * SW_VISIBILITY_NEBULA
	    );
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(" (");
	    i = (int)(bw->vs_zoom * 100.0f);
	    sprintf(ns, "%i%%%%", i);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(") (");
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING("Nebula");
	    SET_COLOR(std_color_num);
	    DRAW_STRING(")");
	}
	else if(obj->loc_type == SW_LOC_TYPE_ASTEROID_FIELD)
	{
	    sprintf(
		ns, 
		"%.2f au",
		obj->scanner_range * legend->ru_to_au * 
		    bw->vs_zoom * SW_VISIBILITY_ASTEROID_FIELD
	    );
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(" (");
	    i = (int)(bw->vs_zoom * 100.0f);
	    sprintf(ns, "%i%%%%", i);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(") (");
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING("AsteroidField");
	    SET_COLOR(std_color_num);
	    DRAW_STRING(")");
	}
	else if(obj->loc_type == SW_LOC_TYPE_PULSAR)
	{
	    sprintf(
		ns, 
		"%.2f au",
		obj->scanner_range * legend->ru_to_au * 
		    bw->vs_zoom * 1.0f
	    );
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(" (");
	    i = (int)(bw->vs_zoom * 100.0f);
	    sprintf(ns, "%i%%%%", i);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(") (");
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING("Pulsar");
	    SET_COLOR(std_color_num);
	    DRAW_STRING(")");
	}
	else
	{
	    sprintf(
		ns, 
		"%.2f au",
		obj->scanner_range * legend->ru_to_au * 
		    bw->vs_zoom * 1.0f
	    );
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(" (");
	    i = (int)(bw->vs_zoom * 100.0f);
	    sprintf(ns, "%i%%%%", i);
	    SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(ns);
	    SET_COLOR(std_color_num);
	    DRAW_STRING(")");
	}

#undef DRAW_STRING
#undef SET_COLOR_BOLD
#undef SET_COLOR
}

/*
 *      Draws subject stats.
 *
 *      Does not clear or refresh the bridge window, the bridge window
 *      is assumed to have been already cleared.
 */
void BridgeDrawSubjectStats(
	CDisplay *display, swt_bridge_struct *bw,
	const int obj_num
)
{
	int i, x, y, hc, wc, std_color_num, sel_wep_num;
	unsigned int width, height;
	float bearing, distance;
	const char *name, *units_str;
	char *s;
	xsw_object_struct *obj, *player_obj;
	xsw_weapon_struct *wep;
	Window w;
	sw_legend_struct *legend = &sw_legend;
	xsw_net_parms_struct *net = &net_parms;


	if(display == NULL)
	    return;

	w = display->root_win;

	width = CWRootWidth(display);
	height = CWRootHeight(display);
	wc = 40;
	hc = (int)height - 5;


	if(DBIsObjectGarbage(obj_num))
	    return;

	obj = xsw_object[obj_num];

	player_obj = net->player_obj_ptr;;
	if(player_obj == NULL)
	    return;


	/* Begin drawing */
	x = (int)width;			/* Right-edge oriented */
	y = 1;
	std_color_num = swt_color.standard;

#define SET_COLOR(_c_)          CWSetColor(display, w, (_c_), 0)
#define SET_COLOR_BOLD(_c_)     CWSetColor(display, w, (_c_), CColorFlagBold)
#define DRAW_STRING(_s_)        CWDrawString(display, w, (_s_))

	/* Subject name */
	name = DBGetFormalNameStr(obj_num);
	s = (char *)realloc(NULL, STRLEN(name) + 80);
	sprintf(s, "%s", name);
	x = (int)width - STRLEN(s) - 2;
	SET_COLOR_BOLD(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    s
	);

	/* Bearing and distance */
	y += 2;
	if(Mu3DInSameSectorPtr(obj, player_obj))
	{
	    bearing = MuCoordinateDeltaVector(
	        obj->x - player_obj->x,
	        obj->y - player_obj->y
	    );
	    distance = Mu3DDistance(
		obj->x - player_obj->x,
		obj->y - player_obj->y,
		obj->z - player_obj->z
	    ) * legend->ru_to_au;
	    units_str = "au";
	}
	else
	{
	    bearing = MuCoordinateDeltaVector( 
		obj->sect_x - player_obj->sect_x,
		obj->sect_y - player_obj->sect_y
	    );
	    distance = Mu3DDistance(
		obj->sect_x - player_obj->sect_x,
		obj->sect_y - player_obj->sect_y,
		obj->sect_z - player_obj->sect_z
	    );
	    units_str = "sects";
	}
	s = (char *)realloc(s, 40 + 40 + STRLEN(units_str) + 80);
	sprintf(
	    s,
	    "BRG: %.2f'  D: %.2f %s",
	    RADTODEG(bearing),
	    distance,
	    units_str
	);
	x = (int)width - STRLEN(s) - 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "BRG: "
	);
	sprintf(s, "%.2f'", RADTODEG(bearing));
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING("  D: ");
	sprintf(s, "%.2f %s", distance, units_str);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);

	/* Hit points */  
	y += 2;
	i = (int)((obj->hp_max > 0.0f) ?
	    ((float)obj->hp / (float)obj->hp_max * 100.0f) :
	    0.0f
	);
	s = (char *)realloc(s, 40 + 40 + 40 + 80);
	sprintf(
	    s,
	    "HP: %.0f(%.0f) %i%%",
	    obj->hp, obj->hp_max, i
	);
	x = (int)width - STRLEN(s) - 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "HP: "
	);
	sprintf(s, "%.0f", obj->hp);
	if(i <= (int)(SWT_HP_DANGER_COEFF * 100.0f))
	    SET_COLOR_BOLD(swt_color.danger);
	else if(i <= (int)(SWT_HP_WARNING_COEFF * 100.0f))
	    SET_COLOR_BOLD(swt_color.warning);
	else
	    SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING("(");
	sprintf(s, "%.0f", obj->hp_max);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING(") ");
	sprintf(s, "%i%%%%", i);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);

	/* Core power */
	y += 1;
	i = (int)((obj->power_max > 0.0f) ?
	    ((float)obj->power / (float)obj->power_max * 100.0f) :
	    0.0f
	);
	s = (char *)realloc(s, 40 + 40 + 80);
	sprintf(
	    s,
	    "CP: %.0f(%.0f)",
	    obj->power,
	    obj->power_max
	);
	x = (int)width - STRLEN(s) - 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "CP: "
	);
	sprintf(s, "%.0f", obj->power);
	if(i <= (int)(SWT_POWER_DANGER_COEFF * 100.0f))
	    SET_COLOR_BOLD(swt_color.danger);
	else if(i <= (int)(SWT_POWER_WARNING_COEFF * 100.0f))
	    SET_COLOR_BOLD(swt_color.warning);
	else
	    SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING("(");
	sprintf(s, "%.0f", obj->hp_max);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);  
	DRAW_STRING(")");

	/* Shield state and freq */
	y += 1;
	if((obj->shield_state == SW_SHIELD_STATE_UP) &&
	   (obj->power_max > 0.0f)
	)
	{
	    i = (int)((obj->power / obj->power_max) * 100.0f);
	    s = (char *)realloc(s, 40 + 80);
	    sprintf(
		s,
		"SH: %i%%",
		i
	    );
	    x = (int)width - STRLEN(s) - 2;
	    SET_COLOR(std_color_num);
	    CWDrawStringPosition(
		display, w,
		x, y,
		"SH: "
	    );
	    sprintf(s, "%i%%%%", i);
	    if(i <= (int)(SWT_POWER_DANGER_COEFF * 100.0f))
		SET_COLOR_BOLD(swt_color.danger);
	    else if(i <= (int)(SWT_POWER_WARNING_COEFF * 100.0f))
		SET_COLOR_BOLD(swt_color.warning);
	    else
		SET_COLOR_BOLD(std_color_num);
	    DRAW_STRING(s);
	}

	/* Sector */
	y += 1;
	s = (char *)realloc(s, 40 + 40 + 40 + 80);
	sprintf(
	    s,
	    "Sect: %ld %ld %ld",
	    obj->sect_x,
	    obj->sect_y,
	    obj->sect_z
	);
	x = (int)width - STRLEN(s) - 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Sect: "
	);
	sprintf(s, "%ld", obj->sect_x);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(s, "%ld", obj->sect_y);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(s, "%ld", obj->sect_z);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);

	/* Position */
	y += 1;
	s = (char *)realloc(s, 40 + 40 + 40 + 80);
	sprintf(
	    s,
	    "Pos: %.2f %.2f %.2f",
	    obj->x,
	    obj->y,
	    obj->z
	);
	x = (int)width - STRLEN(s) - 2;
	SET_COLOR(std_color_num);
	CWDrawStringPosition(
	    display, w,
	    x, y,
	    "Pos: "
	);
	sprintf(s, "%.2f", obj->x);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(s, "%.2f", obj->y);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);
	SET_COLOR(std_color_num);
	DRAW_STRING(" ");
	sprintf(s, "%.2f", obj->z);
	SET_COLOR_BOLD(std_color_num);
	DRAW_STRING(s);


	/* Selected weapon */
	y += 2;
	while(1)
	{
	    sel_wep_num = obj->selected_weapon;
	    if((sel_wep_num < 0) ||
	       (sel_wep_num >= obj->total_weapons)
	    )
		break;

	    wep = obj->weapons[sel_wep_num];
	    if(wep == NULL)
		break;

	    switch(wep->emission_type)
	    {
	      case SW_WEP_EMISSION_PROJECTILE:
		s = (char *)realloc(
		    s,
		    40 + STRLEN(wep->name) + 40 + 40 + 80
		);
		sprintf(
		    s,
"Wep %i: %s: %i(%i)",
		    sel_wep_num + 1,
		    wep->name,
		    wep->amount, wep->max
		);
		x = (int)width - STRLEN(s) - 2;
		SET_COLOR(std_color_num);
		CWDrawStringPosition(
		    display, w,
		    x, y,
		    "Wep "
		);
		sprintf(s, "%i", sel_wep_num + 1);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(s);
		SET_COLOR(std_color_num);
		DRAW_STRING(": ");
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(wep->name);
		SET_COLOR(std_color_num);
		DRAW_STRING(": ");
		sprintf(s, "%i", wep->amount);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(s);
		SET_COLOR(std_color_num);
		DRAW_STRING("(");
		sprintf(s, "%i", wep->max);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(s);
		SET_COLOR(std_color_num);
		DRAW_STRING(")");  
		break;

	      default:
		i = (int)((obj->power_max > 0.0f) ?
		    ((obj->power / obj->power_max) * 100.0f) :
		    0.0f
		);
		s = (char *)realloc(
		    s,
		    40 + STRLEN(wep->name) + 40 + 40 + 80
		);
		sprintf(
		    s,
"Wep %i: %s: %i%%",
		    sel_wep_num + 1,
		    wep->name,
		    i
		);
		x = (int)width - STRLEN(s) - 2;
		SET_COLOR(std_color_num);
		CWDrawStringPosition(
		    display, w,
		    x, y,
		    "Wep "
		);
		sprintf(s, "%i", sel_wep_num + 1);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(s);
		SET_COLOR(std_color_num);
		DRAW_STRING(": ");
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(wep->name);
		SET_COLOR(std_color_num);
		DRAW_STRING(": ");
		sprintf(s, "%i%%%%", i);
		SET_COLOR_BOLD(std_color_num);
		DRAW_STRING(s);
		break;
	    }

	    break;
	}


	free(s);

#undef DRAW_STRING
#undef SET_COLOR_BOLD
#undef SET_COLOR
}

/*
 *	Draws the object.
 *
 *	The obj specifies the object and is assumed valid.
 *
 *	The x and y specifies the center of the object in window
 *	coordinates.
 */
static void BridgeDrawVSObject(
	CDisplay *display, swt_bridge_struct *bw, Window w,
	xsw_object_struct *camera_obj,
	xsw_object_struct *camera_locked_obj,
	xsw_object_struct *obj,
	const int x, const int y,
	const time_t cur_ms, const time_t cur_sec, const float tc
)
{
	const int type = obj->type;
	const float visibility = DBGetObjectVisibilityPtr(obj);
	char c;
	int i, dx, dy;

	/* Do not draw if visiblity is too low */
	if(visibility < 0.10f)
	    return;

#define SET_COLOR(_c_,_a_)	CWSetColor(display, w, (_c_), (_a_))

	/* Choose character to represent object based on type */
	switch(type)
	{
	  case SW_OBJ_TYPE_AREA:
	    SET_COLOR(
		swt_color.vs_obj_area,
		((obj->shield_state == SW_SHIELD_STATE_UP) ?
		    CColorFlagReverse : 0) |
		CColorFlagBold
	    );
	    c = 'A';
	    break;

	  case SW_OBJ_TYPE_HOME:
	    SET_COLOR(
		swt_color.vs_obj_home,
		((visibility >= 0.5f) ?
		    CColorFlagBold : 0)
	    );
	    c = 'O';
	    break;

	  case SW_OBJ_TYPE_PLAYER:
	  case SW_OBJ_TYPE_CONTROLLED:
	    SET_COLOR(
		swt_color.vs_obj_dynamic,
		((visibility >= 0.5f) ?                     
		    CColorFlagBold : 0)
	    );
	    c = '^';
	    break;

	  case SW_OBJ_TYPE_WEAPON:
	  case SW_OBJ_TYPE_STREAMWEAPON:
	  case SW_OBJ_TYPE_SPHEREWEAPON:
	    SET_COLOR(
		swt_color.vs_obj_weapon,
		((visibility >= 0.5f) ?
		    CColorFlagBold : 0)
	    );
	    c = '*';
	    break;

	  default:
	    SET_COLOR(
		swt_color.vs_obj_static, 
		((visibility >= 0.5f) ?
		    CColorFlagBold : 0)
	    );
	    c = '+';
	    break;
	}
	if(camera_locked_obj == obj)
	{
	    SET_COLOR(
		swt_color.vs_obj_locked_on,
		CColorFlagBold
	    );
	    c = '#';
	}
	/* Draw character representing object */
	CWDrawCharacter(
	    display, w,
	    x, y,
	    c
	);


	/* Draw the heading mark */
	i = (int)(((obj->heading / (2.0f * PI)) * 8) + 0.5f);
	switch(i)
	{
	  case 1:
	    dx = 1; dy = -1;
	    c = '/';
	    break;
	  case 2:
	    dx = 1; dy = 0;
	    c = '-';
	    break;
	  case 3:
	    dx = 1; dy = 1;
	    c = '\\';
	    break;
	  case 4:
	    dx = 0; dy = 1;
	    c = '|';
	    break;
	  case 5:
	    dx = -1; dy = 1;
	    c = '/';
	    break;
	  case 6:
	    dx = -1; dy = 0;
	    c = '-';
	    break;
	  case 7:
	    dx = -1; dy = -1;
	    c = '\\';
	    break;
	  default:
	    dx = 0; dy = -1;
	    c = '|';
	    break;
	}

	if((type == SW_OBJ_TYPE_PLAYER) ||
	   (type == SW_OBJ_TYPE_CONTROLLED) ||
	   (type == SW_OBJ_TYPE_WEAPON) ||
	   (type == SW_OBJ_TYPE_STREAMWEAPON)
	)
	    CWDrawCharacter(   
		display, w,
		x + dx, y + dy,
		c
	    );


#undef SET_COLOR
}

/*
 *	Draws the particle.
 *
 *	The p specifies the particle and is assumed valid.
 *
 *	The x and y specify the center of the particle in window
 *	coordinates.
 */
static void BridgeDrawVSParticle(
	CDisplay *display, swt_bridge_struct *bw, Window w,
	xsw_particle_struct *p,
	const int x, const int y,
	const time_t cur_ms, const time_t cur_sec, const float tc
)
{
	float coeff;

#define SET_COLOR(_c_,_a_)	CWSetColor(display, w, (_c_), (_a_))

	switch(p->type)
	{
	  case XSW_PARTICLE_TYPE_SMOKE:
/* Skip, these are probably too numerous */
	    break;

	  case XSW_PARTICLE_TYPE_EFFECT:
	    switch(p->imgref_num)
	    {
	      case IMGREF_FIRE_30:
	      case IMGREF_FIRE_60:
	      case IMGREF_FIRE_90:
		SET_COLOR(swt_color.vs_particle_fire, CColorFlagDim);
		CWDrawCharacter(
		    display, w,
		    x, y,
		    '.'
		);
		break;
	      case IMGREF_EXPLOSION_SMALL:
	      case IMGREF_EXPLOSION_MEDIUM:
	      case IMGREF_EXPLOSION_LARGE:
		if(p->life_span > 0l)
		    coeff = CLIP(
			(float)(cur_ms - p->birth_time) /
			    (float)p->life_span,
			0.0f, 1.0f
		    );
		else
		    coeff = 0.0f;
		if(coeff < 0.34f)
		    SET_COLOR(
			swt_color.vs_particle_explosion,
			0
		    );
		else if(coeff < 0.65f)
		    SET_COLOR(
			swt_color.vs_particle_explosion,
			CColorFlagDim
		    );
		else
		    SET_COLOR(
			swt_color.vs_particle_explosion,
			CColorFlagDim
		    );
		CWDrawCharacter(
		    display, w,
		    x, y,
		    'X'
		);
		break;
	      case IMGREF_SMOKE_30:
	      case IMGREF_SMOKE_60:
	      case IMGREF_SMOKE_90:
		SET_COLOR(swt_color.vs_particle_smoke, CColorFlagDim);
		CWDrawCharacter(
		    display, w,
		    x, y,
		    '.'
		);
		break;
	    }
	    break;

	  case XSW_PARTICLE_TYPE_SHOCKWAVE:
/* TODO */
	    break;
	}

#undef SET_COLOR
}

/*
 *	Procedure to draw each object in range.
 */
void BridgeDrawVSObjects(
	CDisplay *display, swt_bridge_struct *bw,
	const time_t cur_ms, const time_t cur_sec, const float tc
)
{
	int	i, camera_obj_num,
		x, y,
		vs_x, vs_y,
		x_trans, y_trans, longest_dimension;
	float d, real_to_screen_coeff, camera_scanner_range;
	unsigned int width, height;
	xsw_particle_struct *p;
	xsw_object_struct	*obj,
				*camera_obj,
				*camera_locked_obj;
	Window w;
	xsw_net_parms_struct *net = &net_parms;

	if((display == NULL) || (bw == NULL))
	    return;

	w = display->root_win;

	/* Get referances to camera object */
	camera_obj_num = net->player_obj_num;
	camera_obj = net->player_obj_ptr;

	/* Don't draw if camera object is not available */
	if(camera_obj == NULL)
	    return;

	/* Get the viewscreen's geometry */
	vs_x = 1;
	vs_y = 1;
	width = (unsigned int)MAX(
	    ((int)CWRootWidth(display) - 2),
	    0
	);
	height = (unsigned int)MAX(
	    ((int)CWRootHeight(display) - 1 -
		bw->msg_lines_visible - 1 - 1),
	    0
	);

	/* Get longest screen size dimension */
	longest_dimension = (width > height) ? width : height;

	/* Get scanner range of camera object */
	if(camera_obj->loc_type == SW_LOC_TYPE_NEBULA)
	    camera_scanner_range = camera_obj->scanner_range
		* SW_VISIBILITY_NEBULA;
	else if(camera_obj->loc_type == SW_LOC_TYPE_ASTEROID_FIELD)
	    camera_scanner_range = camera_obj->scanner_range
		* SW_VISIBILITY_ASTEROID_FIELD;
	else
	    camera_scanner_range = camera_obj->scanner_range;

	/* Get object that camera is locked on to */
	camera_locked_obj = DBIsObjectGarbage(camera_obj->locked_on) ?
	    NULL : xsw_object[camera_obj->locked_on];

	/* Set screen translation offset */
	if(camera_obj->velocity_max > 0.0f)
	{
	    const float coeff = camera_obj->velocity /
		camera_obj->velocity_max;
	    const sw_vector_compoent_struct *v = &camera_obj->momentum_vector_compoent;
	    x_trans = -(int)(
		(((float)width * 0.4f) * coeff * v->i) + 0.5f
	    );
	    y_trans = (int)(
		(((float)height * 0.4f) * coeff * v->j) + 0.5f
	    );
	}
	else
	{
	    x_trans = 0;
	    y_trans = -0;
	}

	/* Calculate real units to screen units coefficient */
	d = camera_scanner_range * bw->vs_zoom;
	if(d <= 0.0f)
	    d = 0.01f;
	real_to_screen_coeff = (longest_dimension / 2.0f) / d;


	/* Begin drawing objects in the in range objects list */
	for(i = 0; i < total_inrange_objects; i++)
	{
	    obj = inrange_xsw_object[i];
	    if(obj == NULL)
		continue;

	    if(obj->type <= SW_OBJ_TYPE_GARBAGE)
		continue;

	    if(!Mu3DInSectorPtr(
		obj,
		camera_obj->sect_x,
		camera_obj->sect_y,
		camera_obj->sect_z
	    ))
		continue;

	    x = (int)(
		((obj->x - camera_obj->x) * real_to_screen_coeff) +
		((float)width / 2) + 0.5f
	    );
	    y = (int)(
		(float)height - (
		    (obj->y - camera_obj->y) * real_to_screen_coeff
		) - ((float)height / 2) + 0.5f
	    );

	    x += vs_x + x_trans;
	    y += vs_y + y_trans;

	    /* Off screen check */
	    if((x < (vs_x + 1)) ||
	       (y < (vs_y + 1)) ||
	       (x >= (vs_x + (int)width - 1)) ||
	       (y >= (vs_y + (int)height - 1))
	    )
		continue;

	    BridgeDrawVSObject(
		display, bw, w,
		camera_obj,
		camera_locked_obj,
		obj,
		(int)x, (int)y,
		cur_ms, cur_sec, tc
	    );
	}

	/* Draw particles */
	for(i = 0; i < total_particles; i++)
	{
	    p = xsw_particle[i];
	    if(p == NULL)
		continue;

	    if((p->sect_x != camera_obj->sect_x) ||
	       (p->sect_y != camera_obj->sect_y) ||
	       (p->sect_z != camera_obj->sect_z)
	    )
		continue;

	    x = (int)(
		((p->x - camera_obj->x) * real_to_screen_coeff) +
		((float)width / 2) + 0.5f
	    );
	    y = (int)(
		(float)height - (
		    (p->y - camera_obj->y) * real_to_screen_coeff
		) - ((float)height / 2) + 0.5f
	    );

	    x += vs_x + x_trans;
	    y += vs_y + y_trans;

	    /* Off screen check */
	    if((x < (vs_x + 1)) ||
	       (y < (vs_y + 1)) ||
	       (x >= (vs_x + (int)width - 1)) ||
	       (y >= (vs_y + (int)height - 1))
	    )
		continue;  

	    BridgeDrawVSParticle(
		display, bw, w,
		p,
		(int)x, (int)y,
		cur_ms, cur_sec, tc
	    );
	}

}
 

/*
 *	Redraws messag box.
 */
void BridgeDrawMessages(CDisplay *display, swt_bridge_struct *bw)
{
	int i, n, x, y, xc, yc, wc, hc, len, lines_visible;
	char tmp_char;
	const char *s;
	unsigned int width, height;
	Window w;
	swt_msg_struct *msg;

	if((display == NULL) || (bw == NULL))
	    return;

	w = display->root_win;

	width = CWRootWidth(display);
	height = CWRootHeight(display);
	lines_visible = bw->msg_lines_visible;

	x = 1;
	y = (int)height - 1 - lines_visible;

	wc = x + (int)width - 2;
	hc = y + lines_visible;


	/* Set up i to be the starting message index */
	i = bw->message_scroll_pos;
	if(i < 0)
	    i = 0;

	for(yc = y; yc < hc; yc++, i++)
	{
	    if(i >= bw->total_messages)
		break;

	    msg = bw->message[i];
	    if(msg == NULL)
		continue;

	    s = msg->message;
	    len = (s != NULL) ? (int)strlen(s) : 0;

	    CWSetColor(display, w, msg->color_num, CColorFlagBold);

	    for(xc = x, n = 0; xc < wc; xc++, n++)
	    {
		tmp_char = (n < len) ? s[n] : ' ';
		CWDrawCharacter(
		    display,
		    w,
		    xc, yc,
		    tmp_char
		);
	    }
	}
}

/*
 *	Redraws the status bar on the bridge window.
 */
void BridgeDrawStatusBar(CDisplay *display, swt_bridge_struct *bw)
{
	int xc, yc, wc;
	unsigned int width, height;
	time_t aint_int;
	Window w;
	xsw_net_parms_struct *net = &net_parms;

	if((display == NULL) || (bw == NULL))
	    return;

	w = display->root_win;
	width = CWRootWidth(display);
	height = CWRootHeight(display);
	aint_int = MAX(auto_interval_tune.interval, 1l);

	xc = 1;
	yc = (int)height - 1;
	wc = xc + (int)width - 2;

	/* Begin drawing */

	CWSetColor(display, w, swt_color.status, 0);
	CWDrawCharacter(
	    display, w,
	    xc, yc,
	    '['
	);
	xc++;

	CWSetColor(display, w, swt_color.status_con, CColorFlagBold);
	switch(net->connection_state)
	{
	  case XSW_CON_STATE_CONNECTED:
	    CWDrawStringPosition(
		display, w,
		xc, yc,
		"====="
	    );
	    xc += 5;
	    break;
	  case XSW_CON_STATE_NEGOTIATING:
	    CWDrawStringPosition(
		display, w,
		xc, yc,
		"-> <-"
	    );
	    xc += 5;
	    break;
	  case XSW_CON_STATE_CLOSE_WAIT:
	    CWDrawStringPosition(
		display, w,      
		xc, yc,    
		"-<X>-"
	    );
	    xc += 5;
	    break;
	  default:	/* XSW_CON_STATE_NOT_CONNECTED */
	    CWDrawStringPosition(
		display, w,      
		xc, yc,
		"-< >-"
	    );
	    xc += 5;
	    break;
	}

	CWSetColor(display, w, swt_color.status, 0);
	CWDrawCharacter(
	    display, w,
	    xc, yc,
	    ']'
	);
	xc++;

	if(XSW_CON_STATE_IS_CONNECTED(net))
	{
	    char s[80];

	    CWSetColor(display, w, swt_color.status, 0);
	    sprintf(
		s, "   %s: ",
		(auto_interval_tune.state) ? "AInt" : "Int"
	    );
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);

	    CWSetColor(display, w, swt_color.status, CColorFlagBold);
	    sprintf(
		s, "%ld ms",
		net->net_int
	    );
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);


	    CWSetColor(display, w, swt_color.status, 0);
	    strcpy(s, "   SX: ");
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);

	    CWSetColor(display, w, swt_color.status, CColorFlagBold);
	    sprintf(
		s, "%ld",
		net_stat.sx_ilast * 1000l / aint_int
	    );
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);


	    CWSetColor(display, w, swt_color.status, 0);
	    strcpy(s, "   RX: ");
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);

	    CWSetColor(display, w, swt_color.status, CColorFlagBold);
	    sprintf(
		s, "%ld",
		net_stat.rx_ilast * 1000l / aint_int
	    );
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);


	    CWSetColor(display, w, swt_color.status, 0);
	    strcpy(s, "   Lapse: ");
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);

	    CWSetColor(display, w, swt_color.status, CColorFlagBold);
	    sprintf(
		s, "%ld ms",
		lapsed_millitime
	    );
	    CWDrawStringPosition(display, w, xc, yc, s);
	    xc += STRLEN(s);
	}

	CWSetColor(display, w, swt_color.status, 0);
	while(xc < wc)
	{
	    CWDrawCharacter(
		display, w,
		xc, yc,
		' '
	    );
	    xc++;
	}
}
