#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "net.h"
#include "swterm.h"


void BridgeMapPrompt(swt_bridge_struct *bw, const int mode);
void BridgePromptCB(Prompt *p, const char *value, void *data);

void BridgeWarnWeaponsOffline(
	swt_bridge_struct *bw, int obj_num
);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? (int)strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


/*
 *	Maps the bridge window prompt and sets the prompt_mode.
 */
void BridgeMapPrompt(swt_bridge_struct *bw, const int mode)
{
	char *name = "Prompt:";
	Prompt *p;
        CDisplay *display = NULL;

	if(bw == NULL)
	    return;

        if(total_curses_displays > 0)
            display = curses_display[0];
        if(display == NULL)
            return;

	p = &bw->prompt;

	/* Set global prompt mode */
	bw->prompt_mode = mode;

	/* Change prompt's name */
	switch(bw->prompt_mode)
	{
	  case SWT_PROMPT_MODE_CONNECT:
	    name = "Connect:";
	    break;

          case SWT_PROMPT_MODE_CHANNEL:
            name = "Com Channel:";
            break;

          case SWT_PROMPT_MODE_SHIELD_FREQ:
            name = "Shield Frequency:";
            break;

          case SWT_PROMPT_MODE_WEAPON_FREQ:
            name = "Weapon Frequency:";
            break;

          case SWT_PROMPT_MODE_INTERCEPT:
            name = "Intercept:";
            break;

          case SWT_PROMPT_MODE_MESSAGE:
            name = "Message:";
            break;

          case SWT_PROMPT_MODE_SERVER_CMD:
            name = "Server Command:";
            break;

	  case SWT_PROMPT_MODE_EXIT:
	    name = "Exit?:";
	    break;
	}
	PromptSetLabel(display, p, name);

	/* Change prompt's client data */
	p->activate_data = bw;

	/* Clear prompt's contents */
	PromptSetS(display, p, "");

	/* Map prompt */
	PromptResize(
	    display, p,
            MAX(CWRootWidth(display) - 2, 10),
            1
	);
	p->x = 1;
	p->y = CWRootHeight(display) - 1 - bw->msg_lines_visible - 1;

	PromptMap(display, p);
	PromptSetFocus(display, p, 1);
}

/*
 *	Bridge window's prompt callback handler.
 */
void BridgePromptCB(Prompt *p, const char *value, void *data)
{
	int obj_num;
	xsw_object_struct *obj_ptr;
        CDisplay *display = NULL;
	xsw_net_parms_struct *net = &net_parms;
	swt_bridge_struct *bw = SWT_BRIDGE(data);
	if(bw == NULL)
	    return;

        if(total_curses_displays > 0)
            display = curses_display[0];
        if((display == NULL) ||
           (value == NULL)
	)
            return;

	/* Handle by prompt mode */
	switch(bw->prompt_mode)
	{
	  case SWT_PROMPT_MODE_CONNECT:
	    SWTConnect(value);
	    break;

	  case SWT_PROMPT_MODE_CHANNEL:
	    obj_num = net->player_obj_num;
	    obj_ptr = net->player_obj_ptr;
            if(obj_ptr != NULL)
            {
                obj_ptr->com_channel = (int)((double)atof(value) * 100);
                NetSendSetChannel(obj_num, obj_ptr->com_channel);
	    } 
	    break;

	  case SWT_PROMPT_MODE_SHIELD_FREQ:
            obj_num = net->player_obj_num;
            obj_ptr = net->player_obj_ptr;
            if(obj_ptr != NULL)
            {
                NetSendSetShields(
                    obj_num,
                    obj_ptr->shield_state,
                    atof(value)
                );
	    }
	    break;

	  case SWT_PROMPT_MODE_WEAPON_FREQ:
            player_values.weapon_freq = atof(value);
	    break;

	  case SWT_PROMPT_MODE_INTERCEPT:
            obj_num = net->player_obj_num;
            obj_ptr = net->player_obj_ptr;
            if(obj_ptr != NULL)
            {
                NetSendIntercept(
                    obj_num, 
                    ((*value == '\0') ? "#off" : value) 
                );
            }
	    break;

	  case SWT_PROMPT_MODE_MESSAGE:
            obj_num = net->player_obj_num;
            obj_ptr = net->player_obj_ptr;
            if(obj_ptr != NULL)
            {
                NetSendComMessage(
                    obj_num,
                    -1,
                    obj_ptr->com_channel,
                    value
                );
            }
	    break;

	  case SWT_PROMPT_MODE_SERVER_CMD:
            if(net->connection_state == XSW_CON_STATE_CONNECTED)
            {
                NetSendExec(value);
	    }
	    break;

	  case SWT_PROMPT_MODE_EXIT:
	    if(toupper(value[0]) == 'Y')
	    {
		SWTMainPop(NULL);
	    }
	    break;
	}

	/* Reset global prompt_mode */
	bw->prompt_mode = SWT_PROMPT_MODE_NONE;

	/* Unmap prompt */
	PromptUnmap(display, &bw->prompt);
}


void BridgeWarnWeaponsOffline(
	swt_bridge_struct *bw, int obj_num
)
{
	XSWPrint(
 "Weapons are currently offline, press 'w' to bring them online."
	);
}
