/*
		   Curses Library Wrapper Functions
 */

#ifndef CURSESW_H
#define CURSESW_H

#include <curses.h>
#include "curseswkeymap.h"


#ifdef __cplusplus
extern "C" {
#endif


typedef struct _CDisplay		CDisplay;
typedef struct _CColor			CColor;
typedef struct _CEvent			CEvent;
typedef struct _CEventKey		CEventKey;


/*
 *	Flags:
 */
#define CFlagHasColors			(1 << 2)
#define CFlagCanCreateColors		(1 << 3)
#define CFlagTransparentBackground	(1 << 4)


/*
 *	Types:
 */
#define Window				unsigned long
#define Bool				int


/*
 *	Constants:
 */
#ifndef False
# define False				0
#endif
#ifndef True
# define True				1
#endif
#ifndef None
# define None				0
#endif


/*
 *	Color FLags:
 */
#define CColorFlagBold			(1 << 0)
#define CColorFlagDim			(1 << 1)
#define CColorFlagReverse		(1 << 2)
#define CColorFlagUnderline		(1 << 3)
#define CColorFlagBlink			(1 << 4)


/*
 *	Default Root Window Size:
 */
#define DefaultRootWidth		80
#define DefaultRootHeight		20

/*
 *	CEvent type codes:
 */
#define CNoEvent			0
#define CKeyPress			1
#define CKeyRelease			2


/*
 *	Display:
 */
struct _CDisplay {

	unsigned int	flags;	/* Any of CFlag* */

	/* Colormap */
	CColor		*colormap;
	int		ncolors;	/* Number of colors in the colormap,
					 * which is also the number of color
					 * pairs */

	/* Maximum color intensity value */
	short		color_intensity_max;

	/* Root window */
	Window		root_win;

	/* Size of root window, this can change so calling
	 * functions should NOT use these values!!
	 */
	unsigned int	root_width,
			root_height;

	/* Current window cursor is on */
	Window		cursor_window;
	int		cursor_x,
			cursor_y;

};


/*
 *	Color:
 */
struct _CColor {

	/* Private */

	/* Curses color index (0 to COLORS) */
	short		fg_i, bg_i;

	/* Color value (from 0 to 1000) */
	short		fg_red, fg_green, fg_blue,
			bg_red, bg_green, bg_blue;

};

/*
 *	Key Event:
 */
struct _CEventKey {

	int		key;	/* ASCII value of key */

};

/*
 *	Event:
 */
struct _CEvent {

	int		type;
	CEventKey	ckey;

};


extern CDisplay *CWInit(int argc, char **argv);
extern void CWShutdown(CDisplay *display);

extern Window CWCreateWindow(
	CDisplay *display, Window parent,
	int x, int y,
	unsigned int width, unsigned int height
);

extern unsigned int CWRootWidth(CDisplay *display);
extern unsigned int CWRootHeight(CDisplay *display);

extern int CWMatchColorCoeff(
	CDisplay *display,
	const float fg_red, const float fg_green, const float fg_blue,
	const float bg_red, const float bg_green, const float bg_blue
);

extern void CWSetColor(
        CDisplay *display, Window w,
	const int color_num, const unsigned int flags
);
extern void CWMoveCursor(
	CDisplay *display, Window w,
	const int x, const int y
);

extern void CWDrawCharacter(
	CDisplay *display, Window w,
	const int x, const int y,
	const char c
);
extern void CWDrawString(
	CDisplay *display, Window w,
	const char *s
);
extern void CWDrawStringPosition(
	CDisplay *display, Window w,
	const int x, const int y,
	const char *s
);
extern void CWDrawStringPositionLimited(
	CDisplay *display, Window w,
	const int x, const int y,
	const char *s,
	const int limit
);
extern void CWClearArea(
	CDisplay *display, Window w,
	const int x, const int y,
	const unsigned int width, const unsigned int height
);

extern int CGetNextEvent(CDisplay *display, CEvent *event);

extern void CWSync(CDisplay *display, const unsigned int options);


#ifdef __cplusplus
}
#endif


#endif	/* CURSESW_H */
