/*
			    Prompt Widget
 */

#ifndef PROMPT_H
#define PROMPT_H

#include "cursesw.h"


#ifdef __cplusplus
extern "C" {
#endif


typedef struct _Prompt				Prompt;

struct _Prompt {

	Bool		map_state,
			is_in_focus;
	int		x, y;
	unsigned int	width, height;
	Window		w;
	int		color_num;

	char		*label;

	int		buf_start_pos;	/* Relative to x. */

	char		*buf;
	int		buf_len,
			buf_scroll_pos,
			buf_cursor_pos;

	char		**history;
	int		nhistory_items,
			cur_history_pos;

	void		(*activate_cb)(
		Prompt *,		/* Prompt */
		const char *,		/* Value */
		void *			/* Data */
	);
	void		*activate_data;

};


extern char *PromptGetS(CDisplay *display, Prompt *p);
extern int PromptGetI(CDisplay *display, Prompt *p);
extern long PromptGetL(CDisplay *display, Prompt *p);
extern double PromptGetF(CDisplay *display, Prompt *p);

extern void PromptSetLabel(CDisplay *display, Prompt *p, const char *label);
extern void PromptSetS(CDisplay *display, Prompt *p, const char *s);
extern void PromptSetI(CDisplay *display, Prompt *p, int i);
extern void PromptSetL(CDisplay *display, Prompt *p, long l);
extern void PromptSetF(CDisplay *display, Prompt *p, double d, int decimals);

extern int PromptInit(
	CDisplay *display,
	Prompt *p, Window parent,
	int x, int y,
	unsigned int width, unsigned int height,
	int buf_len,
	const char *label,
	void (*activate_cb)(Prompt *, const char *, void *),
	void *activate_data
);
extern void PromptResize(
	CDisplay *display, Prompt *p,
	unsigned int width, unsigned int height
);
extern void PromptDraw(CDisplay *display, Prompt *p);
extern int PromptManage(CDisplay *display, Prompt *p, CEvent *event);
extern void PromptSetFocus(CDisplay *display, Prompt *p, char focus);
extern void PromptMap(CDisplay *display, Prompt *p);
extern void PromptUnmap(CDisplay *display, Prompt *p);
extern void PromptDestroy(CDisplay *display, Prompt *p);


#ifdef __cplusplus
}
#endif


#endif	/* PROMPT_H */
