#ifndef SWTERM_H
#define SWTERM_H

#include <sys/types.h>

#include "../include/imgrefs.h"
#include "../include/objects.h"
#include "sndrefs.h"

#include "cursesw.h"
#include "prompt.h"
#include "gctl.h"


#ifdef __cplusplus
extern "C" {
#endif


/*
 *	Program name and version:
 */
#define PROG_NAME_FULL			"ShipWars Terminal"
#define PROG_NAME			"SWTerm"
#define PROG_VERSION			"2.5.3"

#define PROG_VERSION_MAJOR		2
#define PROG_VERSION_MINOR		5
#define PROG_VERSION_RELEASE		3


/*
 *	Client type code:
 *
 *	To identify this client to the universe server, sent in
 *	a CS_CODE_LOGIN.
 */
#define XSW_CLIENT_TYPE_CODE		7


/*
 *	Usage message:
 */
#define PROG_USAGE_MESSAGE	"\
Usage: swterm [url] [options]\n\
\n\
    [url] is the URL to the universe that you want to connect to,\n\
    the format is:\n\
\n\
	sw://[name]:[password]@<address>:[port]
\n\
    [options] can be any of the following:\n\
\n\
        -d <dir>                Directory containing the XShipWars data\n\
                                (default is /usr/share/games/xshipwars).\n\
        --no_colors             Disable colors.\n\
\n\
        --debug_keyboard        Prints all key event key values to the\n\
                                message window.\n\
        --help                  Prints (this) help screen and exits.\n\
        --version               Prints version information and exits.\n\
\n"

/*
 *	Copyright information:
 */
#define PROG_COPYRIGHT "\
Copyright (C) 2000-2006 WolfPack Entertainment.\n\
This program is protected by international copyright laws and treaties,\n\
distribution and/or modification of this software in violation of the\n\
GNU Public License is strictly prohibited. Violators will be prosecuted\n\
to the fullest extent of the law."


/*
 *	Default Server Address:
 *
 *      This is the default server address and port to connect to when
 *      no address is given at the command line or in the configuration
 *      file.
 */
#define XSW_DEF_SERVER_ADDRESS		"localhost"
#define XSW_DEF_SERVER_PORT		1701

/*
 *	Default Guest Login Name & Password:
 */
#define XSW_DEF_GUEST_LOGIN_NAME	"Guest"
#define XSW_DEF_GUEST_LOGIN_PASSWORD	"guest"



/*
 *	Default Network Bandwidth (in bytes per second):
 */
#define XSW_DEF_NET_BANDWIDTH		3600	/* 28.8 kbps */


/*
 *	Default network stream interval (in milliseconds):
 */
#define XSW_SWSERV_DEF_UPDATE_INT	1000
#define	XSW_SWSERV_MIN_UPDATE_INT	25
#define XSW_SWSERV_MAX_UPDATE_INT	5000


/*
 *	Bridge standard redraws every this many milliseconds:
 */
#define BRIDGE_STANDARD_REDRAW_INTERVAL		100

/*
 *	Memory is refreshed this many milliseconds:
 */
#define XSW_MEMORY_CLEAN_INTERVAL	150000	/* 2.5 minutes */

/*
 *	Object Outdated Timeout:
 *
 *	Objects that have not been updated by network data from the
 *	server for this many milliseconds are assumed outdated and
 *	will not be drawn.
 *
 *	However, the object will not be recycled.
 */
#define XSW_DEF_OBJECT_OUTDATED_TIME    6000


/*
 *	Particle life span and respawn interval:
 */
#define XSW_PARTICLE_LIFE_SPAN		2000
#define XSW_SPAWN_PARTICLES_INTERVAL	100


/*
 *      Default Program Directories:
 *      
 *      These are directories used by XShipWars, make sure that they actually
 *      exist.  These are only default values, they can be changed in the
 *      XSW_RCFILE or XSW_RCFILE_GLOBAL.
 */
#ifdef __WIN32__
# define DEF_XSW_TOPLEVEL_DIR	CWD_STR
# define DEF_XSW_IMAGES_DIR	CWD_STR"images"
# define DEF_XSW_SOUNDS_DIR	CWD_STR"sounds"
# define DEF_XSW_ETC_DIR	CWD_STR
# define DEF_ETC_DIR		CWD_STR
# define DEF_SW_SERVER_DIR	CWD_STR
#else
# define DEF_XSW_TOPLEVEL_DIR	"/usr/share/games/xshipwars"
# define DEF_XSW_IMAGES_DIR	"/usr/share/games/xshipwars/images"
# define DEF_XSW_SOUNDS_DIR	"/usr/share/games/xshipwars/sounds"
# define DEF_XSW_ETC_DIR	"/usr/share/games/xshipwars/etc"
# define DEF_ETC_DIR		"/etc"
# define DEF_SW_SERVER_DIR	"/home/swserv"
#endif


/*
 *      Default configuration file:
 */
#define XSW_DEF_CFG_FILE		"xsw.ini"

/* 
 *	Default log file:
 */
#define XSW_DEF_LOG_FILE		"xsw.log"

/* 
 *      Default OCSNs file location:
 *
 *      Default parent is /usr/games/xshipwars/etc/.
 */
#define DEF_XSW_OCSN_FILE               "default.ocsn"

/*
 *      Default sound scheme file location:
 *
 *      Default parent is /usr/games/xshipwars/sounds/.
 */
#define DEF_XSW_SOUND_SCHEME_FILE       "default.ss"





/*
 *	Throttle Modes:
 */
#define XSW_THROTTLE_MODE_NORMAL	0
#define XSW_THROTTLE_MODE_BIDIRECTIONAL	1
#define XSW_THROTTLE_MODE_INCREMENTAL	2


/*
 *	Image Code:
 */
#define XSW_IMG_DIALOG_ICON_INFO	10
#define XSW_IMG_DIALOG_ICON_WARNING	11
#define XSW_IMG_DIALOG_ICON_ERROR	12
#define XSW_IMG_DIALOG_ICON_QUESTION	13


/*
 *	Fake GUI types:
 */
#define bool_t		Bool
#define key_state_t	unsigned int
typedef struct { int i; } image_t;
#define pixmap_t	unsigned long
#define win_t		unsigned long
#define drawable_t	unsigned long


/*          
 *      Pixel type:
 */         
#define font_t		void
#define pixel_t         unsigned long
typedef struct {
	u_int8_t	a, r, g, b;
} WColorStruct;


/*
 *	Object Flags:
 */
#define XSW_OBJ_FLAG_ON_FIRE		(1 << 8)
#define XSW_OBJ_FLAG_SMOKE_TRAIL	(1 << 9)


/*
 *	HP & Power Level Coefficients:
 *
 *	These coefficients are used to determine when levels of hp
 *	and power are considered at "warning" or "danger" levels in
 *	order to determine the color of the display and when to
 *	announce it to the player.
 */
#define SWT_HP_WARNING_COEFF		0.58
#define SWT_HP_DANGER_COEFF		0.30
#define SWT_POWER_WARNING_COEFF		SWT_HP_WARNING_COEFF
#define SWT_POWER_DANGER_COEFF		SWT_HP_DANGER_COEFF
#define SWT_ANTIMATTER_WARNING_COEFF	0.40
#define SWT_ANTIMATTER_DANGER_COEFF	0.20

/*
 *	Runlevel:
 */
extern int runlevel;


/*
 *      Current time since midnight in milliseconds:
 */
extern time_t cur_millitime;

/*      
 *      Current systime seconds:
 */
extern time_t cur_systime;


/*
 *      Lapsed MilliTime:
 *
 *      In milliseconds, the time it took the previous loop to execute.
 *      This is compared to CYCLIC_MILLITIME_LAPSE (#defined in reality.h)
 *      to adjust for time lost in the last loop.
 */
extern time_t lapsed_millitime;

/*
 *      Time Compensation:
 *
 *      This value is always in the range of 1.0 to <big number>.
 *      It is used as the coefficent to various momentum and movement
 *      calculations to compensate for lost time in the previous
 *      loop.
 */
extern float time_compensation;


/*
 *      Global debug values:
 *
 *      Used for run time debugging.
 */
#define DEBUG_LEVEL_NONE		0
#define DEBUG_LEVEL_ALL			1
#define DEBUG_LEVEL_MEMORY		2
#define DEBUG_LEVEL_NETWORK		3
typedef struct {

	int		level;      /* One of DEBUG_LEVEL_* */  
	float		val;

} xsw_debug_struct;
extern xsw_debug_struct debug;


/*
 *	Options:
 */
typedef struct _swt_option_struct	xsw_option_struct;

/*
 *	CPU Priorities:
 */
#define XSW_CPU_PRIORITY_HIGHEST        0       /* usleep(0) */
#define XSW_CPU_PRIORITY_HIGH           1       /* usleep(1000) */
#define XSW_CPU_PRIORITY_MEDIUM         2       /* usleep(8000) (default) */
#define XSW_CPU_PRIORITY_LOWEST         3       /* usleep(16000) */

/*
 *	Units:
 */
#define XSW_UNITS_XSW			0	/* XSW Internal units */
#define XSW_UNITS_METRIC		1	/* AstroMetric */
#define XSW_UNITS_ENGLISH		2	/* Universal English */

/*
 *      Show Formal Object Labels:
 */
#define XSW_SHOW_FORMAL_LABEL_NEVER	0
#define XSW_SHOW_FORMAL_LABEL_AS_NEEDED	1
#define XSW_SHOW_FORMAL_LABEL_ALWAYS	2


struct _swt_option_struct {

	/* Configuration file */
	int		cfg_version_major,
			cfg_version_minor,
			cfg_version_release;

	/* CPU Priority (one of XSW_CPU_PRIORITY_*) */
	int		cpu_priority;                 

	/* Units, one of XSW_UNITS_* */
	int		units;

	/* Log options */
	bool_t		log_client,
			log_net,
			log_errors;

	/* Debug keyboard */
	bool_t		debug_kb;

	/* Energy saver mode (1 for on, 0 for off) */
	bool_t		energy_saver_mode;

	/* Throttle mode, one of XSW_THROTTLE_MODE_* */
	int		throttle_mode;

	/* Show formal labels (one of XSW_SHOW_FORMAL_LABEL_*) */
	int		show_formal_label;

	bool_t		show_fires,
			show_smoke_trails;

	int		max_particles;

	/* Net bandwidth in bytes per second */
	unsigned long	net_bandwidth;

	/* Do not show objects that have not been updated for this
	 * many milliseconds
	 */
	time_t		net_object_outdated_time;

	/* Show network errors (undicipherable network data)? */
	bool_t		show_net_errors;

	/* Show server error message dialogs? */
	bool_t		show_server_errors;

	/* Update objects locally by predicting movement patterns */
	bool_t		local_updates;

	/* Sound level, one of XSW_SOUNDS_* */
#define XSW_SOUNDS_NONE         0
#define XSW_SOUNDS_EVENTS       1
#define XSW_SOUNDS_ENGINE       2
#define XSW_SOUNDS_ALL          3
	int		sounds;
	bool_t		music;     /* 0 for off, 1 for on */


	/* Scanner range affected by enviroment? (set true, no cheating!) */
	bool_t		scanner_limiting;

	/* Notify when objects come into or leave scanner range? */
	bool_t		notify_scanner_contacts;

	bool_t		auto_map_eco_win;

	bool_t		save_on_exit;
};

extern xsw_option_struct	option;


/*
 *	GUI capopff:
 */
typedef struct {

	int		i;

#define OSW_KEY_STATE_ALT		(1 << 0)
#define OSW_KEY_STATE_CTRL		(1 << 1)
#define OSW_KEY_STATE_SHIFT		(1 << 2)

#define OSW_KEY_STATE_CAPS_LOCK		(1 << 5)
#define OSW_KEY_STATE_NUM_LOCK		(1 << 6)
#define OSW_KEY_STATE_SCROLL_LOCK	(1 << 7)
	key_state_t	key_state;

} osw_gui_struct;
extern osw_gui_struct osw_gui[1];

/*
 *	Curses IO display structure array:
 */
extern CDisplay **curses_display;
extern int total_curses_displays;


/*
 *	Directories:
 */
typedef struct _swt_dname_struct	xsw_dname_struct;

struct _swt_dname_struct {

	char		startup[PATH_MAX],	/* Startup dir */
			toplevel[PATH_MAX],	/* XSW's toplevel dir */
			etc[PATH_MAX],		/* Configuration dir */
			images[PATH_MAX],	/* Images dir */
			sounds[PATH_MAX],	/* Sounds dir */
			downloads[PATH_MAX];	/* Screen shot dir */

};

extern xsw_dname_struct	dname;


/*
 *	Files:
 */
typedef struct _swt_fname_struct	xsw_fname_struct;

struct _swt_fname_struct {

	char		cfg[NAME_MAX + PATH_MAX],
			ocsn[NAME_MAX + PATH_MAX],
			imgrefs[NAME_MAX + PATH_MAX],
			sndrefs[NAME_MAX + PATH_MAX],
			log[NAME_MAX + PATH_MAX];

};

extern xsw_fname_struct	fname;


/*
 *	Next event scheduals:
 *
 *	Marks the next time in milliseconds that something
 *	is to be performed.
 */
typedef struct _swt_next_struct		xsw_next_struct;

struct _swt_next_struct {

	time_t		update_check,
			viewscreen,		/* Redraw viewscreen */
			bridge_std_redraw,	/* Standard bridge graphics redraw */
			memory_clean,		/* Reclaim memory */
			lplayer_pos_send,	/* Next send player position */
			spawn_particles,
			wormhole_enter;

};

extern xsw_next_struct	next;



/*
 *	Player Values:
 */
typedef struct {

	char		weapons_online;
	float		weapon_freq;
	float		weapon_fire_heading;
	float		weapon_yield;

	time_t		auto_destruct_start_int,	/* Seconds */
			auto_destruct_last_sec_left;	/* Seconds */

} xsw_player_values_struct;

extern xsw_player_values_struct player_values;


/*
 *	Game Controller:
 */
extern xsw_gctl_struct	*xsw_gctl;


/*
 *   Auto interval tuning:
 *
 *      Relies on information in loadstat.
 */
typedef struct {

	bool_t		state;

	/* Re-tuned every this many milliseconds */
#define AINT_DEF_TUNE_INT       1000
	time_t		interval;	/* Always 1000 ms (every second) */

	/* Next update in milliseconds */
	time_t		next;

} xsw_auto_interval_tune_struct;

extern xsw_auto_interval_tune_struct auto_interval_tune;


/*
 *	Colors:
 */
typedef struct _swt_color_struct	swt_color_struct;
typedef struct _swt_color_struct	xsw_color_struct;

struct _swt_color_struct {

	int		standard,
			warning,
			danger,

			title,
			messages,
			status,
			status_con,

			/* Viewscreen object colors */
			vs_obj_static,
			vs_obj_dynamic,
			vs_obj_weapon,
			vs_obj_home,
			vs_obj_area,
                        vs_obj_locked_on,

			/* Viewscreen particle colors */
			vs_particle_fire,
			vs_particle_smoke,
			vs_particle_explosion;

	/* Capoffs */
	WColorStruct	bp_hostile_cv,
			bp_friendly_cv,
			bp_unknown_cv,
			vs_label_fg_cv,
			vs_label_bg_cv;

};

extern swt_color_struct	swt_color;
#define xsw_color		swt_color


/*
 *	Fonts:
 */
typedef struct {

	/* All purpose fonts */
	font_t  *std,
		*std_bold;

	/* Bridge console panel fonts */
	font_t  *vs_standard,
		*console_heading,
		*console_standard,
		*console_message;

} xsw_font_struct;
extern xsw_font_struct xsw_font;


/* Bridge console panel codes */ 
#define BW_PANEL_ALL			0

#define BW_PANEL_P1			1
#define BW_PANEL_P2			2
#define BW_PANEL_P3			3
#define BW_PANEL_P4			4

#define BW_PANEL_S1			5
#define BW_PANEL_S2			6
#define BW_PANEL_S3			7 

#define BW_PANEL_PNAME			20
#define BW_PANEL_PSHIELDFREQ		21
#define BW_PANEL_PWEAPONFREQ		22
#define BW_PANEL_PCOMCHANNEL		23
#define BW_PANEL_PSCORE			24

#define BW_PANEL_PHULL			30
#define BW_PANEL_PPOWER			31
#define BW_PANEL_PSHIELDS		32
#define BW_PANEL_PVIS			33
#define BW_PANEL_PDMGCTL		34

#define BW_PANEL_SNAME			40
#define BW_PANEL_SEMPIRE		41
#define BW_PANEL_SHULL			42
#define BW_PANEL_SPOWER			43
#define BW_PANEL_SSHIELDS		44
#define BW_PANEL_SVIS			45
#define BW_PANEL_SBEARING		46
#define BW_PANEL_SDISTANCE		47
#define BW_PANEL_SDMGCTL		48

#define BW_PANEL_PINAME			50
#define BW_PANEL_PWLOCK			51
#define BW_PANEL_PWEAPONS		52

#define BW_PANEL_SINAME			60
#define BW_PANEL_SWLOCK			61
#define BW_PANEL_SWEAPONS		62

#define BW_PANEL_SANTIMATTER		63
#define BW_PANEL_SHEADING		64	/* And coordinates */

#define BW_PANEL_PTHROTTLE		70
#define BW_PANEL_PVELOCITY		71
#define BW_PANEL_PANTIMATTER		72
#define BW_PANEL_PENGINESTATE		73
#define BW_PANEL_PHEADING		74	/* And coordinates */
#define BW_PANEL_PTHRUSTVECTOR		75


/*
 *	Bridge:
 */
typedef struct _swt_bridge_struct	swt_bridge_struct;
#define SWT_BRIDGE(p)			((swt_bridge_struct *)(p))
typedef struct _swt_bridge_struct	xsw_bridge_win_struct;
#define XSW_BRIDGE_WIN(p)		((xsw_bridge_win_struct *)(p))
typedef struct _swt_msg_struct		swt_msg_struct;

struct _swt_msg_struct {
	char		*message;
	int		color_num;
};

struct _swt_bridge_struct {

	bool_t		map_state,
			has_focus;
	int		x, y;
	unsigned int	width, height;

	unsigned int	toplevel;

	/* Viewscreen zoom */
#define VS_ZOOM_MIN	0.01
#define VS_ZOOM_MAX	1.00
	float		vs_zoom;


	/* Screen display type */
#define BRIDGE_SCREEN_TYPE_MAIN_MENU    0
#define BRIDGE_SCREEN_TYPE_VIEWSCREEN   1
#define BRIDGE_SCREEN_TYPE_MESSAGES     2       /* All messages */
#define BRIDGE_SCREEN_TYPE_HELP         3
	int		screen_type;


	/* Prompt */
	Prompt		prompt;

	/* Prompt mode */
#define SWT_PROMPT_MODE_NONE		0
#define SWT_PROMPT_MODE_CONNECT		1
#define SWT_PROMPT_MODE_CHANNEL		2
#define SWT_PROMPT_MODE_SHIELD_FREQ	3
#define SWT_PROMPT_MODE_WEAPON_FREQ	4
#define SWT_PROMPT_MODE_INTERCEPT	5
#define SWT_PROMPT_MODE_MESSAGE		6
#define SWT_PROMPT_MODE_SERVER_CMD	7
#define SWT_PROMPT_MODE_EXIT		8
	int		prompt_mode;


	/* Messages */
	swt_msg_struct		**message;
	int			total_messages;
	int			msg_lines_visible;
	int			message_scroll_pos;

	/* Fake XSW cap offs */
	image_t			*vs_weapon_image;
	pixmap_t		vs_weapon_buf;

};

extern swt_bridge_struct	*swt_bridge;
#define xsw_bridge_win		swt_bridge


/*
 *	Economy Window:
 */
typedef struct _swt_eco_win_struct	swt_eco_win_struct;
typedef struct _swt_eco_win_struct      xsw_eco_win_struct;

struct _swt_eco_win_struct {

	bool_t		map_state,
			has_focus;
	int		x, y;
	unsigned int	width, height;

};

extern swt_eco_win_struct	*xsw_ecow;


/*
 *	Dialog:
 */
typedef struct _swt_dialog_struct	dialog_struct;

struct _swt_dialog_struct {

	bool_t		map_state,
			has_focus;
	int		x, y;
	unsigned int	width, height;

};

extern dialog_struct	err_dw,
			info_dw;


/*
 *	ShipWars Universe Legend:
 */
extern sw_legend_struct	sw_legend;


/*
 *	Scanner Contact:
 *
 *      List record of objects in scanner range.
 *
 *      Used to keep track of which objects have just entered
 *      or left sensor range.
 */
typedef struct _swt_scanner_contact_struct	xsw_scanner_contact_struct;

struct _swt_scanner_contact_struct {

	int		obj_num;

};

extern xsw_scanner_contact_struct **scanner_contact;
extern int total_scanner_contacts;


/*
 *	ImgRefs:
 */
extern "C" imgref_struct        **xsw_imgref;
extern "C" int          total_imgrefs;


/*
 *	SndRefs:
 */
extern "C" sndref_struct	**xsw_sndref;
extern "C" int		total_sndrefs;


/*
 *	Objects:
 */
extern xsw_object_struct	**xsw_object;
extern int		total_objects;  

/*
 *	In Range Objects:
 *
 *	List of pointers to objects that are in range of the player
 *	object.
 */
extern xsw_object_struct	**inrange_xsw_object;
extern int		total_inrange_objects;


/* In autointerval.cpp */
extern void AutoIntervalTune(void);


/* In bridge.cpp */
extern swt_bridge_struct *BridgeNew(void);
extern void BridgeDraw(swt_bridge_struct *bw, int portion);
extern void BridgeDrawSchedualed(
	swt_bridge_struct *bw,
	const time_t cur_ms, const time_t cur_sec, const float tc
);
extern int BridgeManage(swt_bridge_struct *bw, CEvent *event);
extern void BridgeSetFocus(swt_bridge_struct *bw, int focus);
extern void BridgeMap(swt_bridge_struct *bw);
extern void BridgeUnmap(swt_bridge_struct *bw);
extern void BridgeDelete(swt_bridge_struct *bw);


/* In bridgedraw.cpp */
extern void BridgeDrawTitle(CDisplay *display, swt_bridge_struct *bw);
extern void BridgeDrawHelp(CDisplay *display, swt_bridge_struct *bw);

extern void BridgeDrawClearVS(CDisplay *display, swt_bridge_struct *bw);
extern void BridgeDrawPlayerStats(
	CDisplay *display, swt_bridge_struct *bw,
	const int obj_num
);
extern void BridgeDrawSubjectStats(
	CDisplay *display, swt_bridge_struct *bw,
	const int obj_num
);

extern void BridgeDrawVSObjects(
	CDisplay *display, swt_bridge_struct *bw,
	const time_t cur_ms, const time_t cur_sec, const float tc
);

extern void BridgeDrawMessages(CDisplay *display, swt_bridge_struct *bw);
extern void BridgeDrawStatusBar(CDisplay *display, swt_bridge_struct *bw);


/* In bridgeop.cpp */
extern void BridgeMapPrompt(swt_bridge_struct *bw, const int mode);
extern void BridgePromptCB(Prompt *p, const char *value, void *data);

extern void BridgeWarnWeaponsOffline(
	xsw_bridge_win_struct *bw, int obj_num
);


/* In db.cpp */
extern int DBIsObjectGarbage(int object_num);

extern xsw_object_struct **DBAllocObjectPointers(
	xsw_object_struct **cur_obj_ptrs,
	int num_objects
);
extern xsw_object_struct *DBAllocObject(void);

extern int DBCreateObject(
	int imageset,
	int type,
	int owner,
	long sect_x, long sect_y, long sect_z,
	float x, float y, float z,
	float heading, float pitch, float bank
);
extern int DBCreateExplicitObject(
	int obj_num,
	int imageset,
	int type,
	int owner,
	long sect_x, long sect_y, long sect_z,
	float x, float y, float z,
	float heading, float pitch, float bank
);
extern void DBDeleteObject(xsw_object_struct *obj_ptr);
extern void DBDeleteAllObjects(void);
extern void DBRecycleObject(int object_num); 

extern void DBReclaim(void);


/* In dbinrange.cpp */
extern void DBInRangeUpdate(int object_num);
extern int DBInRangeAdd(int ref_obj, int tar_obj, char check_range);
extern void DBInRangeDelete(xsw_object_struct *obj_ptr);
extern void DBInRangeDeleteAll(void);


/* In dbutil.cpp */
extern int DBCreateObjectEconomyProduct(
	int obj_num,
	const char *product_name,
	int ocs_code,
	sw_credits_t sell_price,
	sw_credits_t buy_price,
	float product_amount,
	float product_max
);

extern void DBObjectTractor(int obj_num, int tar_obj_num);

extern int DBValidateObjectName(const char *name);
extern int DBValidateObjectPassword(const char *password);

extern float DBGetObjectVisibility(int obj_num);
extern float DBGetObjectVisibilityPtr(xsw_object_struct *obj_ptr);

extern float DBGetObjectEffectiveScannerRange(int obj_num);
extern float DBGetObjectEffectiveScannerRangePtr(xsw_object_struct *obj);

extern char DBIsObjectInEffectiveScannerRange(
	int obj_num, int tar_obj_num,
	float scanner_range
);
extern char DBIsObjectInEffectiveScannerRangePtr(
	xsw_object_struct *obj, xsw_object_struct *tar_obj,
	float scanner_range
);

extern int DBGetTopObjectNumber(void);
extern int DBGetObjectNumByPtr(xsw_object_struct *obj_ptr);
extern const char *DBGetFormalNameStr(int object_num);
extern const char *DBGetObjectVectorName(float theta);
extern float DBGetObjectVisibility(int object_num);
extern float DBGetObjectVisibilityPtr(xsw_object_struct *obj_ptr);

extern void DBSetPlayerObject(int object_num);


/* main.cpp */
extern void SWTHandleSignal(int s);
extern void SWTResetTimmers(const time_t cur_ms, const time_t cur_sec);
extern void SWTReclaimGlobalMemory(bool_t print_response);

extern void SWTOpenOCSNs(const char *path);
extern void SWTOpenImgRefs(const char *path);
extern void SWTOpenSndRefs(const char *path);

extern void SWTUpdateBackgroundMusic(void);

extern int SWTConnect(const char *arg);
extern void SWTDisconnect(void);
extern void SWTRefresh(void);
extern void SWTDamageNotify(
        const int obj_num,
        const float prev_hp, const float hp,
        const float prev_power, const float power,
        const int shield_state
);
extern void SWTHitNotify(
	const int src_obj_num,
	const int tar_obj_num,
	const int owner_obj_num,
	const float dmg_structure,
	const float dmg_shield,
	const float bearing
);
extern void SWTDestroyNotify(
	const int src_obj_num, const int tar_obj_num,
	const int owner_obj, const int reason
);

extern int SWTManageGUI(void *data);
extern int SWTManageSound(void *data);
extern int SWTManageNet(void *data);

extern int SWTInit(int argc, char **argv);
extern void SWTMainIteration(void *data);
extern void SWTMain(void *data);
extern void SWTMainPop(void *data);
extern void SWTMainPopAll(void *data);
extern void SWTShutdown(void);


/* In rengine.cpp */
extern void REngObjectStopped(
        xsw_object_struct *player_obj, xsw_object_struct *obj,
        const time_t cur_ms, const time_t cur_sec, const float tc
);
extern int REngInit(void);
extern void REngManage(
	xsw_object_struct *player_obj,
	time_t cur_ms, time_t cur_sec, float tc
);
extern void REngShutdown(void);


/* In scanner.cpp */
extern int ScIsAllocated(int entry_num);

extern int ScAddObjectInContact(xsw_object_struct *obj_ptr);
extern void ScRemoveObjectFromContact(xsw_object_struct *obj_ptr);

extern int ScIsObjectInContact(xsw_object_struct *obj_ptr);

extern void ScDelete(int entry_num);
extern void ScDeleteAll(void);

extern void ScReclaim(void);

extern int ScHandleContacts(int object_num);


/* In timming.cpp */
extern time_t MilliTime(void);


/* In swtmesg.cpp */
extern void MesgAdd(const char *s, const int color_num);


/* In timming.cpp */
extern time_t MilliTime(void);


/* In xsweqv.cpp */
extern void XSWDialog(
	const char *title, int icon_img_code, const char *msg,
	unsigned int toplevel
);
extern int XSWGetResponse(
	const char *title, int icon_img_code, const char *msg,
	unsigned int btn_flags, unsigned int toplevel
);
extern void XSWPrint(const char *msg);
extern void XSWPrintWarning(const char *msg);
extern void XSWPrintError(const char *msg);
extern void XSWPrintBold(const char *msg);
extern void XSWPrintDebugStderr(int error_level);
extern void XSWPrintHelpStdout(void);
extern void XSWPrintVersionStdout(void);
extern void XSWPrintHelp(void);

extern void XSWLog(const char *s);

extern void XSWOpenOCSNs(const char *path);
extern void XSWOpenImgRefs(const char *path);
extern void XSWOpenSndRefs(const char *path);

extern int XSWConnect(const char *url);
extern void XSWDisconnect(void);
extern void XSWUpdateBackgroundMusic(void);

extern void XSWDoObjectChangeSector(int obj_num);

extern void XSWDamageNotify(
        const int obj_num,
        const float prev_hp, const float hp,
        const float prev_power, const float power,
        const int shield_state
);
extern void XSWHitNotify(
	int src_obj_num,        /* Object that hit tar_obj_num */
	int tar_obj_num,        /* Object that got hit */
	int owner_obj_num,      /* Owner of src_obj_num */
	float structure_damage,
	float shield_damage,
	float bearing          /* Target to source, in radians */
);
extern void XSWDestroyNotify(
	int src_obj_num,        /* Object that caused destruction */
	int tar_obj_num,        /* Object that was destroyed */
	int owner_obj_num,      /* Owner of src_obj_num */     
	int reason              /* Reason code (sent from server) */
);


/* In xswcapoff.cpp */
extern void BridgeWinSetTitle(
	xsw_bridge_win_struct *bw,
	const char *address,
	const char *title,
	const char *player_name
);
extern void BridgeWinDrawPanel(
	int obj_num,
	int panel_detail
);
extern void BridgeWinDrawAll(xsw_bridge_win_struct *bw);
extern void BridgeWinMap(xsw_bridge_win_struct *bw);

extern void EcoWinUpdateCredits(
	xsw_eco_win_struct *ew,
	const int obj_num
);
extern void EcoWinAddProduct(
	xsw_eco_win_struct *ew,
	const int proprietor_obj_num,
	xsw_ecoproduct_struct product
);
extern void EcoWinQueueDraw(xsw_eco_win_struct *ew);
extern void EcoWinMap(xsw_eco_win_struct *ew);

extern imgref_struct *ImgRefGetPtr(
        imgref_struct **imgrefs_list, const int nimgrefs,
        const int imgref_num
);
extern int ImgRefIsLoaded(imgref_struct *imgref);
extern int ImgRefLoad(
        imgref_struct ***imgrefs_list, int *nimgrefs,
        const int imgref_num
);
extern void ImgRefUnload(
        imgref_struct **imgrefs_list, const int nimgrefs,
        const int imgref_num                             
);
extern int ImgRefFileOpen(
	imgref_struct ***imgrefs_list, int *nimgrefs,
	const char *filename
);

extern void ScannerUpdateLabels(xsw_bridge_win_struct *bw, int obj_num);

extern void OSWDestroyImage(image_t **image);

extern void VSDrawUpdateWeaponLabel(
	image_t **image,
	pixmap_t pixmap
);
extern void VSDrawUpdateNetstatsLabel(
	image_t **image,
	pixmap_t pixmap
);

extern int VSLabelAdd(
	const char *text,
	const WColorStruct *fg,
	const WColorStruct *bg,
	font_t *font,          
	int obj_num, xsw_object_struct *obj
);
extern void VSLabelDelete(int n);
extern void VSLabelDeleteByObjectPtr(xsw_object_struct *obj);
extern void VSLabelDeleteAll(void);


#ifdef __cplusplus
}
#endif


#endif	/* SWTERM_H */
