#include <stdlib.h>
#include <string.h>

#include "../include/string.h"

#include "swterm.h"


void MesgAdd(const char *s, const int color_num);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? (int)strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)



/*
 *	Appends the message to the bridge window's list of messages.
 *
 *	If the specified message is too long to fit on one line then
 *	more than one line may be appended.
 */
void MesgAdd(const char *s, const int color_num)
{
	int	i, m, len,
		msg_pos, msg_len,
		longest_line, nmsgs, max_columns;
	CDisplay *display = NULL;
	swt_msg_struct **msg_list, *msg, *msg_prev;
	swt_bridge_struct *bw = swt_bridge;

	if(bw == NULL)
	    return;

	if(total_curses_displays > 0)
	    display = curses_display[0];
	if(display == NULL)
	    return;

	/* Message lines are numbered from bottom to top, so message
	 * index 0 would be the last line
	 */

	max_columns = (unsigned int)MAX(((int)bw->width - 2), 1);
	msg_list = bw->message;
	nmsgs = bw->total_messages;

	/* Get the total length of the new message */
	msg_len = STRLEN(s);

	longest_line = 0;

	/* Add the specified message to the list, adding multiple lines
	 * as needed
	 */
	msg_pos = 0;
	while(msg_pos < msg_len)
	{
	    /* Shift messages "up" */
	    m = nmsgs - 1;
	    for(i = 0; i < m; i++)
	    {
		msg = msg_list[i];
		msg_prev = msg_list[i + 1];

		free(msg->message);
		msg->message = STRDUP(msg_prev->message);

		msg->color_num = msg_prev->color_num;

		/* Update the longest line */
		len = STRLEN(msg->message);
		if(len > longest_line)
		    longest_line = len;
	    }

	    /* Get the first message position (highest index) in the
	     * message list, this where we are going to put the current
	     * segment of the specified message to
	     */
	    msg = msg_list[m];

	    len = MIN(STRLEN(s + msg_pos), max_columns);

	    free(msg->message);
	    msg->message = (char *)malloc((len + 1) * sizeof(char));
	    memcpy(msg->message, s + msg_pos, len);
	    msg->message[len] = '\0';

	    msg->color_num = color_num;

	    /* Update the longest line */
	    if(len > longest_line)
		longest_line = len;

	    /* Increment the specified message's current position
	     * based on how much of it we added on this message
	     */
	    msg_pos += len;
	}

	/* Redraw as needed */
	if(bw->map_state)
	    BridgeDrawMessages(display, bw);     
}
