#include <stdio.h>
#include "../include/reality.h"


float ConvertRUToAU(sw_units_struct *units, float x);
float ConvertRUToSU(sw_units_struct *units, float x);

float ConvertVelocityRUPCToRUPS(sw_units_struct *units, float x);
float ConvertVelocityRUPCToAUPS(sw_units_struct *units, float x);


/*
 *	Convert XSW real units to Astronomical units.
 */
float ConvertRUToAU(sw_units_struct *units, float x)
{
	if(units == NULL)
	    return(0.0f);
	else
	    return(x * units->ru_to_au);
}

/*
 *	Convert XSW real units to screen units.
 */
float ConvertRUToSU(sw_units_struct *units, float x)
{
	return(x * 1000.0f);
}


/*
 *	Convert velocity in XSW real units per cycle (quantom)
 *	XSW real units per second.
 */
float ConvertVelocityRUPCToRUPS(sw_units_struct *units, float x)
{
	return(
	    x * (1000.0f / (float)SW_CYCLE_LAPSE_MS)
	);
}

/*
 *      Convert velocity in XSW real units per cycle (quantom)
 *      to Astronomical units per second.
 */
float ConvertVelocityRUPCToAUPS(sw_units_struct *units, float x)
{
	if(units == NULL)
	    return(0.0f);
	else
	    return(
		(x * units->ru_to_au) *	/* Convert real units to meters. */
		(1000.0f / (float)SW_CYCLE_LAPSE_MS)
	    );
}




