#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/unvutil.h"
#include "../include/unvfile.h"

#include "net.h"
#include "swterm.h"


void XSWDialog(
	const char *title, int icon_img_code, const char *msg,
	unsigned int toplevel
);
int XSWGetResponse(
	const char *title, int icon_img_code, const char *msg,
	unsigned int btn_flags, unsigned int toplevel
);
void XSWPrint(const char *msg);
void XSWPrintWarning(const char *msg);
void XSWPrintError(const char *msg);
void XSWPrintBold(const char *msg);
void XSWPrintDebugStderr(int error_level);
void XSWPrintHelpStdout(void);
void XSWPrintVersionStdout(void);
void XSWPrintHelp(void);

void XSWLog(const char *s);

void XSWOpenOCSNs(const char *path);
void XSWOpenImgRefs(const char *path);
void XSWOpenSndRefs(const char *path);

int XSWConnect(const char *url);
void XSWDisconnect(void);

void XSWUpdateBackgroundMusic(void);

void XSWDoObjectChangeSector(int obj_num);

void XSWDamageNotify(
	const int obj_num,
	const float prev_hp, const float hp,
	const float prev_power, const float power,
	const int shield_state
);
void XSWHitNotify(
	const int src_obj_num,	/* Object that hit tar_obj_num */
	const int tar_obj_num,	/* Object that got hit */
	const int owner_obj_num,	/* Owner of src_obj_num */
	const float dmg_structure,
	const float dmg_shield,
	const float bearing	/* Target to source, in radians */
);
void XSWDestroyNotify(
	const int src_obj_num,	/* Object that caused destruction */
	const int tar_obj_num,	/* Object that was destroyed */
	const int owner_obj_num,	/* Owner of src_obj_num */
	const int reason	/* Reason code (sent from server) */
);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? (int)strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : True)


void XSWDialog(
	const char *title, int icon_img_code, const char *msg,
	unsigned int toplevel
)
{
	return;
}

int XSWGetResponse(
	const char *title, int icon_img_code, const char *msg,
	unsigned int btn_flags, unsigned int toplevel
)
{
	return(0);
}

void XSWPrint(const char *msg)
{
	if(curses_display > 0)
	    MesgAdd(msg, swt_color.messages);
	else
	    fprintf(stdout, "%s\n", msg);
}

void XSWPrintWarning(const char *msg)
{
	if(curses_display > 0)
	    MesgAdd(msg, swt_color.warning);
	else
	    fprintf(stderr, "%s\n", msg);
}

void XSWPrintError(const char *msg)
{
	if(curses_display > 0)
	    MesgAdd(msg, swt_color.danger);
	else
	    fprintf(stderr, "%s\n", msg);
}

void XSWPrintBold(const char *msg)
{
	if(curses_display > 0)
	    MesgAdd(msg, swt_color.messages);
	else
	    fprintf(stdout, "%s\n", msg);
}


void XSWPrintDebugStderr(int error_level)
{

}

/*
 *	Print usage of this program to stdout.
 */
void XSWPrintHelpStdout(void)
{
	fputs(PROG_USAGE_MESSAGE, stdout);
}

/*
 *	Print version of this program to stdout.
 */
void XSWPrintVersionStdout(void)
{
	fputs(
	    PROG_NAME_FULL " Version " PROG_VERSION "\n"
	    PROG_COPYRIGHT "\n",
	    stdout
	);
}

#if 0
void XSWPrintHelp(void);
#endif


void XSWLog(const char *s)
{
	return;
}

void XSWOpenOCSNs(const char *path)
{
	SWTOpenOCSNs(path);
}

void XSWOpenImgRefs(const char *path)
{
	SWTOpenImgRefs(path);
}

void XSWOpenSndRefs(const char *path)
{
	SWTOpenSndRefs(path);
}


int XSWConnect(const char *url)
{
	return(SWTConnect(url));
}

void XSWDisconnect(void)
{
	SWTDisconnect();
}

void XSWUpdateBackgroundMusic(void)
{
	SWTUpdateBackgroundMusic();
}

void XSWDoObjectChangeSector(int obj_num)
{
	xsw_object_struct *obj;
	xsw_net_parms_struct *net = &net_parms;

	if(DBIsObjectGarbage(obj_num))
	    return;

	obj = xsw_object[obj_num];

	/* Is this the player object? */
	if(obj == net->player_obj_ptr)
	{                                  
	    ScHandleContacts(obj_num);
	    DBReclaim();
	    DBInRangeUpdate(obj_num);
	}
}

void XSWDamageNotify(
	const int obj_num,
	const float prev_hp, const float hp,
	const float prev_power, const float power,
	const int shield_state
)
{
	SWTDamageNotify(
	    obj_num,
	    prev_hp, hp,
	    prev_power, power,
	    shield_state
	);
}

void XSWHitNotify(
	const int src_obj_num,	/* Object that hit tar_obj_num */
	const int tar_obj_num,	/* Object that got hit */
	const int owner_obj_num,	/* Owner of src_obj_num */
	const float dmg_structure,
	const float dmg_shield,
	const float bearing	/* Target to source, in radians */
)
{
	SWTHitNotify(
	    src_obj_num,
	    tar_obj_num,
	    owner_obj_num,
	    dmg_structure,
	    dmg_shield,
	    bearing
	);
}

void XSWDestroyNotify(
	const int src_obj_num,        /* Object that caused destruction */
	const int tar_obj_num,        /* Object that was destroyed */
	const int owner_obj_num,      /* Owner of src_obj_num */     
	const int reason              /* Reason code (sent from server) */
)
{
	SWTDestroyNotify(
	    src_obj_num, tar_obj_num,
	    owner_obj_num, reason
	);
}
